@@ -1,7 +1,50 @@
 package Beam::Wire;
-$Beam::Wire::VERSION = '1.019';
+our $VERSION = '1.020';
 # ABSTRACT: Lightweight Dependency Injection Container
 
+#pod =head1 SYNOPSIS
+#pod
+#pod     # wire.yml
+#pod     captain:
+#pod         class: Person
+#pod         args:
+#pod             name: Malcolm Reynolds
+#pod             rank: Captain
+#pod     first_officer:
+#pod         $class: Person
+#pod         name: Zoë Alleyne Washburne
+#pod         rank: Commander
+#pod
+#pod     # script.pl
+#pod     use Beam::Wire;
+#pod     my $wire = Beam::Wire->new( file => 'wire.yml' );
+#pod     my $captain = $wire->get( 'captain' );
+#pod     print $captain->name; # "Malcolm Reynolds"
+#pod
+#pod =head1 DESCRIPTION
+#pod
+#pod Beam::Wire is a configuration module and a dependency injection
+#pod container. In addition to complex data structures, Beam::Wire configures
+#pod and creates plain old Perl objects.
+#pod
+#pod A dependency injection (DI) container creates an inversion of control:
+#pod Instead of manually creating all the dependent objects (also called
+#pod "services") before creating the main object that we actually want, a DI
+#pod container handles that for us: We describe the relationships between
+#pod objects, and the objects get built as needed.
+#pod
+#pod Dependency injection is sometimes called the opposite of garbage
+#pod collection. Rather than ensure objects are destroyed in the right order,
+#pod dependency injection makes sure objects are created in the right order.
+#pod
+#pod Using Beam::Wire in your application brings great flexibility,
+#pod allowing users to easily add their own code to customize how your
+#pod project behaves.
+#pod
+#pod For an L<introduction to the Beam::Wire service configuration format,
+#pod see Beam::Wire::Help::Config|Beam::Wire::Help::Config>.
+#pod
+#pod =cut
 
 use strict;
 use warnings;
@@ -17,6 +60,14 @@ use Types::Standard qw( :all );
 use Data::Dumper;
 use constant DEBUG => $ENV{BEAM_WIRE_DEBUG};
 
+#pod =attr file
+#pod
+#pod The path of the file where services are configured (typically a YAML
+#pod file). The file's contents should be a single hashref. The keys are
+#pod service names, and the values are L<service
+#pod configurations|Beam::Wire::Help::Config>.
+#pod
+#pod =cut
 
 has file => (
     is      => 'ro',
@@ -29,6 +80,13 @@ has file => (
     },
 );
 
+#pod =attr dir
+#pod
+#pod The directory path to use when searching for inner container files.
+#pod Defaults to the directory which contains the file specified by the
+#pod L<file attribute|/file>.
+#pod
+#pod =cut
 
 has dir => (
     is      => 'ro',
@@ -43,6 +101,18 @@ has dir => (
     },
 );
 
+#pod =attr config
+#pod
+#pod The raw configuration data. By default, this data is loaded by
+#pod L<Config::Any|Config::Any> using the file specified by the L<file attribute|/file>.
+#pod
+#pod See L<Beam::Wire::Help::Config for details on what the configuration
+#pod data structure looks like|Beam::Wire::Help::Config>.
+#pod
+#pod If you don't want to load a file, you can specify this attribute in the
+#pod Beam::Wire constructor.
+#pod
+#pod =cut
 
 has config => (
     is      => 'ro',
@@ -57,6 +127,13 @@ sub _build_config {
     return $self->_load_config( $self->file );
 }
 
+#pod =attr services
+#pod
+#pod A hashref of cached services built from the L<configuration|/config>. If
+#pod you want to inject a pre-built object for other services to depend on,
+#pod add it here.
+#pod
+#pod =cut
 
 has services => (
     is      => 'ro',
@@ -71,6 +148,14 @@ sub _build_services {
     return $services;
 }
 
+#pod =attr meta_prefix
+#pod
+#pod The character that begins a meta-property inside of a service's C<args>. This
+#pod includes C<$ref>, C<$path>, C<$method>, and etc...
+#pod
+#pod The default value is C<$>. The empty string is allowed.
+#pod
+#pod =cut
 
 has meta_prefix => (
     is      => 'ro',
@@ -78,6 +163,47 @@ has meta_prefix => (
     default => sub { q{$} },
 );
 
+#pod =method get
+#pod
+#pod     my $service = $wire->get( $name );
+#pod     my $service = $wire->get( $name, %overrides )
+#pod
+#pod The get method resolves and returns the service named C<$name>, creating
+#pod it, if necessary, with L<the create_service method|/create_service>.
+#pod
+#pod C<%overrides> is an optional list of name-value pairs. If specified,
+#pod get() will create an new, anonymous service that extends the named
+#pod service with the given config overrides. For example:
+#pod
+#pod     # test.pl
+#pod     use Beam::Wire;
+#pod     my $wire = Beam::Wire->new(
+#pod         config => {
+#pod             foo => {
+#pod                 args => {
+#pod                     text => 'Hello, World!',
+#pod                 },
+#pod             },
+#pod         },
+#pod     );
+#pod
+#pod     my $foo = $wire->get( 'foo', args => { text => 'Hello, Chicago!' } );
+#pod     print $foo; # prints "Hello, Chicago!"
+#pod
+#pod This allows you to create factories out of any service, overriding service
+#pod configuration at run-time.
+#pod
+#pod If C<$name> contains a slash (C</>) character (e.g. C<foo/bar>), the left
+#pod side (C<foo>) will be used as the name of an inner container, and the
+#pod right side (C<bar>) is a service inside that container. For example,
+#pod these two lines are equivalent:
+#pod
+#pod     $bar = $wire->get( 'foo/bar' );
+#pod     $bar = $wire->get( 'foo' )->get( 'bar' );
+#pod
+#pod Inner containers can be nested as deeply as desired (C<foo/bar/baz/fuzz>).
+#pod
+#pod =cut
 
 sub get {
     my ( $self, $name, %override ) = @_;
@@ -124,6 +250,17 @@ sub get {
     return $service;
 }
 
+#pod =method set
+#pod
+#pod     $wire->set( $name => $service );
+#pod
+#pod The set method configures and stores the specified C<$service> with the
+#pod specified C<$name>. Use this to add or replace built services.
+#pod
+#pod Like L<the get() method, above|/get>, C<$name> can contain a slash (C</>)
+#pod character to traverse through nested containers.
+#pod
+#pod =cut
 
 ## no critic ( ProhibitAmbiguousNames )
 # This was named set() before I started using Perl::Critic, and will
@@ -138,6 +275,15 @@ sub set {
     return;
 }
 
+#pod =method get_config
+#pod
+#pod     my $conf = $wire->get_config( $name );
+#pod
+#pod Get the config with the given C<$name>. Like L<the get() method,
+#pod above|/get>, C<$name> can contain slash (C</>) characters to traverse
+#pod through nested containers.
+#pod
+#pod =cut
 
 sub get_config {
     my ( $self, $name ) = @_;
@@ -150,6 +296,30 @@ sub get_config {
     return $self->config->{$name};
 }
 
+#pod =method normalize_config
+#pod
+#pod     my $out_conf = $self->normalize_config( $in_conf );
+#pod
+#pod Normalize the given C<$in_conf> into to hash that L<the create_service
+#pod method|/create_service> expects. This method allows a service to be
+#pod defined with prefixed meta-names (C<$class> instead of C<class>) and
+#pod the arguments specified without prefixes.
+#pod
+#pod For example, these two services are identical.
+#pod
+#pod     foo:
+#pod         class: Foo
+#pod         args:
+#pod             fizz: buzz
+#pod
+#pod     foo:
+#pod         $class: Foo
+#pod         fizz: buzz
+#pod
+#pod The C<$in_conf> must be a hash, and must already pass L<an is_meta
+#pod check|/is_meta>.
+#pod
+#pod =cut
 
 sub normalize_config {
     my ( $self, $conf ) = @_;
@@ -176,6 +346,107 @@ sub normalize_config {
     return \%out_conf;
 }
 
+#pod =method create_service
+#pod
+#pod     my $service = $wire->create_service( $name, %config );
+#pod
+#pod Create the service with the given C<$name> and C<%config>. Config can
+#pod contain the following keys:
+#pod
+#pod =over 4
+#pod
+#pod =item class
+#pod
+#pod The class name of an object to create. Can be combined with C<method>,
+#pod and C<args>. An object of any class can be created with Beam::Wire.
+#pod
+#pod =item args
+#pod
+#pod The arguments to the constructor method. Used with C<class> and
+#pod C<method>. Can be a simple value, or a reference to an array or
+#pod hash which will be dereferenced and passed in to the constructor
+#pod as a list.
+#pod
+#pod If the C<class> consumes the L<Beam::Service role|Beam::Service>,
+#pod the service's C<name> and C<container> will be added to the C<args>.
+#pod
+#pod =item method
+#pod
+#pod The method to call to create the object. Only used with C<class>.
+#pod Defaults to C<"new">.
+#pod
+#pod This can also be an array of hashes which describe a list of methods
+#pod that will be called on the object. The first method should create the
+#pod object, and each subsequent method can be used to modify the object. The
+#pod hashes should contain a C<method> key, which is a string containing the
+#pod method to call, and optionally C<args> and C<return> keys. The C<args>
+#pod key works like the top-level C<args> key, above. The optional C<return>
+#pod key can have the special value C<"chain">, which will use the return
+#pod value from the method as the value for the service (L<The tutorial shows
+#pod examples of this|Beam::Wire::Help::Config/Multiple Constructor
+#pod Methods>).
+#pod
+#pod If an array is used, the top-level C<args> key is not used.
+#pod
+#pod =item value
+#pod
+#pod The value of this service. Can be a simple value, or a reference to an
+#pod array or hash. This value will be simply returned by this method, and is
+#pod mostly useful when using container files.
+#pod
+#pod C<value> can not be used with C<class> or C<extends>.
+#pod
+#pod =item config
+#pod
+#pod The path to a configuration file, relative to L<the dir attribute|/dir>.
+#pod The file will be read with L<Config::Any>, and the resulting data
+#pod structure returned.
+#pod
+#pod =item extends
+#pod
+#pod The name of a service to extend. The named service's configuration will
+#pod be merged with this configuration (via L<the merge_config
+#pod method|/merge_config>).
+#pod
+#pod This can be used in place of the C<class> key if the extended configuration
+#pod contains a class.
+#pod
+#pod =item with
+#pod
+#pod Compose a role into the object's class before creating the object. This
+#pod can be a single string, or an array reference of strings which are roles
+#pod to combine.
+#pod
+#pod This uses L<Moo::Role|Moo::Role> and L<the create_class_with_roles
+#pod method|Role::Tiny/create_class_with_roles>, which should work with any
+#pod class (as it uses L<the Role::Tiny module|Role::Tiny> under the hood).
+#pod
+#pod This can be used with the C<class> key.
+#pod
+#pod =item on
+#pod
+#pod Attach an event handler to a L<Beam::Emitter subclass|Beam::Emitter>. This
+#pod is an array of hashes of event names and handlers. A handler is made from
+#pod a service reference (C<$ref> or an anonymous service), and a subroutine to
+#pod call on that service (C<$sub>).
+#pod
+#pod For example:
+#pod
+#pod     emitter:
+#pod         class: My::Emitter
+#pod         on:
+#pod             - my_event:
+#pod                 $ref: my_handler
+#pod                 $sub: on_my_event
+#pod
+#pod This can be used with the C<class> key.
+#pod
+#pod =back
+#pod
+#pod This method uses L<the parse_args method|/parse_args> to parse the C<args> key,
+#pod L<resolving references|resolve_ref> as needed.
+#pod
+#pod =cut
 
 sub create_service {
     my ( $self, $name, %service_info ) = @_;
@@ -241,6 +512,9 @@ sub create_service {
     }
     else {
         my @args = $self->parse_args( $name, @service_info{"class","args"} );
+        if ( $service_info{class}->can( 'DOES' ) && $service_info{class}->DOES( 'Beam::Service' ) ) {
+            push @args, name => $name, container => $self;
+        }
         $service = $service_info{class}->$method( @args );
     }
 
@@ -278,6 +552,22 @@ sub create_service {
     return $service;
 }
 
+#pod =method merge_config
+#pod
+#pod     my %merged = $wire->merge_config( %config );
+#pod
+#pod If C<%config> contains an C<extends> key, merge the extended config together
+#pod with this one, returning the merged service configuration. This works recursively,
+#pod so a service can extend a service that extends another service just fine.
+#pod
+#pod When merging, hashes are combined, with the child configuration taking
+#pod precedence. The C<args> key is handled specially to allow a hash of
+#pod args to be merged.
+#pod
+#pod The configuration returned is a safe copy and can be modified without
+#pod effecting the original config.
+#pod
+#pod =cut
 
 sub merge_config {
     my ( $self, %service_info ) = @_;
@@ -303,6 +593,23 @@ sub merge_config {
     return %service_info;
 }
 
+#pod =method parse_args
+#pod
+#pod     my @args = $wire->parse_args( $for_name, $class, $args );
+#pod
+#pod Parse the arguments (C<$args>) for the given service (C<$for_name>) with
+#pod the given class (C<$class>).
+#pod
+#pod C<$args> can be an array reference, a hash reference, or a simple
+#pod scalar. The arguments will be searched for references using L<the
+#pod find_refs method|/find_refs>, and then a list of arguments will be
+#pod returned, ready to pass to the object's constructor.
+#pod
+#pod Nested containers are handled specially by this method: Their inner
+#pod references are not resolved by the parent container. This ensures that
+#pod references are always relative to the container they're in.
+#pod
+#pod =cut
 
 sub parse_args {
     my ( $self, $for, $class, $args ) = @_;
@@ -347,6 +654,23 @@ sub parse_args {
     return @args;
 }
 
+#pod =method find_refs
+#pod
+#pod     my @resolved = $wire->find_refs( $for_name, @args );
+#pod
+#pod Go through the C<@args> and recursively resolve any references and
+#pod services found inside, returning the resolved result. References are
+#pod identified with L<the is_meta method|/is_meta>.
+#pod
+#pod If a reference contains a C<$ref> key, it will be resolved by L<the
+#pod resolve_ref method|/resolve_ref>. Otherwise, the reference will be
+#pod treated as an anonymous service, and passed directly to L<the
+#pod create_service method|/create_service>.
+#pod
+#pod This is used when L<creating a service|create_service> to ensure all
+#pod dependencies are created first.
+#pod
+#pod =cut
 
 sub find_refs {
     my ( $self, $for, @args ) = @_;
@@ -384,6 +708,24 @@ sub find_refs {
     return wantarray ? @out : $out[-1];
 }
 
+#pod =method is_meta
+#pod
+#pod     my $is_meta = $wire->is_meta( $ref_hash, $root );
+#pod
+#pod Returns true if the given hash reference describes some kind of
+#pod Beam::Wire service. This is used to identify service configuration
+#pod hashes inside of larger data structures.
+#pod
+#pod A service hash reference must contain at least one key, and must either
+#pod contain a L<prefixed|/meta_prefix> key that could create or reference an
+#pod object (one of C<class>, C<extends>, C<config>, C<value>, or C<ref>) or,
+#pod if the C<$root> flag exists, be made completely of unprefixed meta keys
+#pod (as returned by L<the get_meta_names method|/get_meta_names>).
+#pod
+#pod The C<$root> flag is used by L<the get method|/get> to allow unprefixed
+#pod meta keys in the top-level hash values.
+#pod
+#pod =cut
 
 sub is_meta {
     my ( $self, $arg, $root ) = @_;
@@ -413,6 +755,14 @@ sub is_meta {
     return;
 }
 
+#pod =method get_meta_names
+#pod
+#pod     my %meta_keys = $wire->get_meta_names;
+#pod
+#pod Get all the possible service keys with the L<meta prefix|/meta_prefix> already
+#pod attached.
+#pod
+#pod =cut
 
 sub get_meta_names {
     my ( $self ) = @_;
@@ -435,6 +785,64 @@ sub get_meta_names {
     return wantarray ? %meta : \%meta;
 }
 
+#pod =method resolve_ref
+#pod
+#pod     my @value = $wire->resolve_ref( $for_name, $ref_hash );
+#pod
+#pod Resolves the given dependency from the configuration hash (C<$ref_hash>)
+#pod for the named service (C<$for_name>). Reference hashes contain the
+#pod following keys:
+#pod
+#pod =over 4
+#pod
+#pod =item $ref
+#pod
+#pod The name of a service in the container. Required.
+#pod
+#pod =item $path
+#pod
+#pod A data path to pick some data out of the reference. Useful with C<value>
+#pod and C<config> services.
+#pod
+#pod     # container.yml
+#pod     bounties:
+#pod         value:
+#pod             malcolm: 50000
+#pod             zoe: 35000
+#pod             simon: 100000
+#pod
+#pod     captain:
+#pod         class: Person
+#pod         args:
+#pod             name: Malcolm Reynolds
+#pod             bounty:
+#pod                 $ref: bounties
+#pod                 $path: /malcolm
+#pod
+#pod =item $call
+#pod
+#pod Call a method on the referenced object and use the resulting value. This
+#pod may be a string, which will be the method name to call, or a hash with
+#pod C<$method> and C<$args>, which are the method name to call and the
+#pod arguments to that method, respectively.
+#pod
+#pod     captain:
+#pod         class: Person
+#pod         args:
+#pod             name: Malcolm Reynolds
+#pod             location:
+#pod                 $ref: beacon
+#pod                 $call: get_location
+#pod             bounty:
+#pod                 $ref: news
+#pod                 $call:
+#pod                     $method: get_bounty
+#pod                     $args:
+#pod                         name: mreynolds
+#pod
+#pod =back
+#pod
+#pod =cut
 
 sub resolve_ref {
     my ( $self, $for, $arg ) = @_;
@@ -481,6 +889,20 @@ sub resolve_ref {
     return @ref;
 }
 
+#pod =method fix_refs
+#pod
+#pod     my @fixed = $wire->fix_refs( $for_name, @args );
+#pod
+#pod Similar to L<the find_refs method|/find_refs>. This method searches
+#pod through the C<@args> and recursively fixes any reference paths to be
+#pod absolute. References are identified with L<the is_meta
+#pod method|/is_meta>.
+#pod
+#pod This is used by L<the get_config method|/get_config> to ensure that the
+#pod configuration can be passed directly in to L<the create_service
+#pod method|create_service>.
+#pod
+#pod =cut
 
 sub fix_refs {
     my ( $self, $container_name, @args ) = @_;
@@ -512,6 +934,13 @@ sub fix_refs {
 }
 
 
+#pod =method new
+#pod
+#pod     my $wire = Beam::Wire->new( %attributes );
+#pod
+#pod Create a new container.
+#pod
+#pod =cut
 
 sub BUILD {
     my ( $self ) = @_;
@@ -626,9 +1055,19 @@ sub validate {
     return $error_count;
 }
 
+#pod =head1 EXCEPTIONS
+#pod
+#pod If there is an error internal to Beam::Wire, an exception will be thrown. If there is an
+#pod error with creating a service or calling a method, the exception thrown will be passed-
+#pod through unaltered.
+#pod
+#pod =head2 Beam::Wire::Exception
+#pod
+#pod The base exception class
+#pod
+#pod =cut
 
 package Beam::Wire::Exception;
-$Beam::Wire::Exception::VERSION = '1.019';
 use Moo;
 with 'Throwable';
 use Types::Standard qw( :all );
@@ -639,9 +1078,13 @@ has error => (
     isa => Str,
 );
 
+#pod =head2 Beam::Wire::Exception::Constructor
+#pod
+#pod An exception creating a Beam::Wire object
+#pod
+#pod =cut
 
 package Beam::Wire::Exception::Constructor;
-$Beam::Wire::Exception::Constructor::VERSION = '1.019';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -652,9 +1095,13 @@ has attr => (
     required => 1,
 );
 
+#pod =head2 Beam::Wire::Exception::Config
+#pod
+#pod An exception loading the configuration file.
+#pod
+#pod =cut
 
 package Beam::Wire::Exception::Config;
-$Beam::Wire::Exception::Config::VERSION = '1.019';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -680,9 +1127,13 @@ has '+error' => (
     },
 );
 
+#pod =head2 Beam::Wire::Exception::Service
+#pod
+#pod An exception with service information inside
+#pod
+#pod =cut
 
 package Beam::Wire::Exception::Service;
-$Beam::Wire::Exception::Service::VERSION = '1.019';
 use Moo;
 use Types::Standard qw( :all );
 extends 'Beam::Wire::Exception';
@@ -698,9 +1149,13 @@ has file => (
     isa         => Maybe[InstanceOf['Path::Tiny']],
 );
 
+#pod =head2 Beam::Wire::Exception::NotFound
+#pod
+#pod The requested service or configuration was not found.
+#pod
+#pod =cut
 
 package Beam::Wire::Exception::NotFound;
-$Beam::Wire::Exception::NotFound::VERSION = '1.019';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 
@@ -714,9 +1169,21 @@ has '+error' => (
     },
 );
 
+#pod =head2 Beam::Wire::Exception::InvalidConfig
+#pod
+#pod The configuration is invalid:
+#pod
+#pod =over 4
+#pod
+#pod =item *
+#pod
+#pod Both "value" and "class" or "extends" are defined. These are mutually-exclusive.
+#pod
+#pod =back
+#pod
+#pod =cut
 
 package Beam::Wire::Exception::InvalidConfig;
-$Beam::Wire::Exception::InvalidConfig::VERSION = '1.019';
 use Moo;
 extends 'Beam::Wire::Exception::Service';
 use overload q{""} => sub {
@@ -744,7 +1211,7 @@ Beam::Wire - Lightweight Dependency Injection Container
 
 =head1 VERSION
 
-version 1.019
+version 1.020
 
 =head1 SYNOPSIS
 
@@ -922,7 +1389,7 @@ contain the following keys:
 =item class
 
 The class name of an object to create. Can be combined with C<method>,
-and C<args>.
+and C<args>. An object of any class can be created with Beam::Wire.
 
 =item args
 
@@ -931,6 +1398,9 @@ C<method>. Can be a simple value, or a reference to an array or
 hash which will be dereferenced and passed in to the constructor
 as a list.
 
+If the C<class> consumes the L<Beam::Service role|Beam::Service>,
+the service's C<name> and C<container> will be added to the C<args>.
+
 =item method
 
 The method to call to create the object. Only used with C<class>.
@@ -1218,12 +1688,16 @@ Al Newkirk <anewkirk@ana.io>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Bruce Armstrong Kent Fredric
+=for stopwords Ben Moon Bruce Armstrong Doug Bell Kent Fredric
 
 =over 4
 
 =item *
 
+Ben Moon <guiltydolphin@gmail.com>
+
+=item *
+
 Bruce Armstrong <bruce@armstronganchor.net>
 
 =item *
@@ -1232,6 +1706,14 @@ Bruce Armstrong <bruce@fortressofgeekdom.org>
 
 =item *
 
+Doug Bell <doug@preaction.me>
+
+=item *
+
+Doug Bell <preaction@users.noreply.github.com>
+
+=item *
+
 Kent Fredric <kentnl@cpan.org>
 
 =back
